"""Functions that read and write gzipped files.

The user of the file doesn't have to worry about the compression.
Seeks are allowed, but are efficient only for files compressed by
the dictzip utility (which adhere to the gzip format).  Files may
be concatenated to overcome dictzip's 1.8 Gb size limit."""

# based on Andrew Kuchling's minigzip.py distributed with the zlib module
# Random access read/write and seek-whence support by Gerton Lunter, 3/3/07

import struct
import sys
import time
import bisect
import zlib
import __builtin__

__all__ = ["GzipFile","open"]

FTEXT, FHCRC, FEXTRA, FNAME, FCOMMENT = 1, 2, 4, 8, 16

READ, WRITE = 1, 2

SEEK_SET, SEEK_CUR, SEEK_END = 0, 1, 2

def U32(i):
    """Return i as an unsigned integer, assuming it fits in 32 bits.

    If it's >= 2GB when viewed as a 32-bit unsigned int, return a long.
    """
    if i < 0:
        i += 1L << 32
    return i

def LOWU32(i):
    """Return the low-order 32 bits of an int, as a non-negative int."""
    return i & 0xFFFFFFFFL

def write32(output, value):
    output.write(struct.pack("<l", value))

def write32u(output, value):
    # The L format writes the bit pattern correctly whether signed
    # or unsigned.
    output.write(struct.pack("<L", value))

def read32(input):
    return struct.unpack("<l", input.read(4))[0]

def open(filename, mode="rb", compresslevel=9, buffersize=None, chunksize=58315):
    """Shorthand for GzipFile(filename, mode, compresslevel, buffersize, chunksize).

    The filename argument is required; mode defaults to 'rb', compresslevel
    defaults to 9, buffersize to None (no random access points) and chunksize to 58315
    (good compression but slowish seeks; irrelevant without random access points)

    """
    return GzipFile(filename, mode, compresslevel, buffersize=buffersize, chunksize=chunksize)

class GzipFile:
    """The GzipFile class simulates most of the methods of a file object with
    the exception of the readinto() and truncate() methods.

    """

    myfileobj = None

    def __init__(self, filename=None, mode=None,
                 compresslevel=9, fileobj=None, buffersize=None, chunksize=58315):
        """Constructor for the GzipFile class.

        At least one of fileobj and filename must be given a
        non-trivial value.

        The new class instance is based on fileobj, which can be a regular
        file, a StringIO object, or any other object which simulates a file.
        It defaults to None, in which case filename is opened to provide
        a file object.

        When fileobj is not None, the filename argument is only used to be
        included in the gzip file header, which may includes the original
        filename of the uncompressed file.  It defaults to the filename of
        fileobj, if discernible; otherwise, it defaults to the empty string,
        and in this case the original filename is not included in the header.

        The mode argument can be any of 'r', 'rb', 'a', 'ab', 'w', or 'wb',
        depending on whether the file will be read or written.  The default
        is the mode of fileobj if discernible; otherwise, the default is 'rb'.
        Be aware that only the 'rb', 'ab', and 'wb' values should be used
        for cross-platform portability.

        The compresslevel argument is an integer from 1 to 9 controlling the
        level of compression; 1 is fastest and produces the least compression,
        and 9 is slowest and produces the most compression.  The default is 9.

        A nonzero buffersize argument instructs GZip to do buffered compression,
        allowing it to include a dictzip field in the header with flush points
        for random access.  The chunksize argument determines the distance between
        flush points; smaller values means faster random access but lower
        compression.  The default value is close to maximum compression.

        """

        # guarantee the file is opened in binary mode on platforms
        # that care about that sort of thing
        if mode and 'b' not in mode:
            mode += 'b'
        if fileobj is None:
            fileobj = self.myfileobj = __builtin__.open(filename, mode or 'rb')
        if filename is None:
            if hasattr(fileobj, 'name'): filename = fileobj.name
            else: filename = ''
        if mode is None:
            if hasattr(fileobj, 'mode'): mode = fileobj.mode
            else: mode = 'rb'

        if mode[0:1] == 'r':
            self.mode = READ
            # Set flag indicating start of a new member
            self._new_member = True
            # Set flag indicating normal gzip format
            self.dictzip = False
            self.extrabuf = ""
            self.extrasize = 0
            self.filename = filename

        elif mode[0:1] == 'w' or mode[0:1] == 'a':
            self.mode = WRITE
            if buffersize: self.dictzip = True
            else: self.dictzip = False
            self.compresslevel = compresslevel
            self.chunksize = chunksize
            self.buffersize = buffersize
            # dictzip's default chunk size of 58315 is too conservative
            if chunksize > 65400:
                raise IOError, "Chunk size " + str(chunksize) + " is too large; maximum is 65400"
            if self.dictzip and buffersize // chunksize > 32764:
                raise IOError, "Buffer size " + str(buffersize) + " is too large; may result in too many chunks"

            self._init_write(filename)

        else:
            raise IOError, "Mode " + mode + " not supported"

        self.fileobj = fileobj
        self.offset = 0

        if self.mode == WRITE:
            if self.dictzip:
                # intialize write buffer
                self.writebuf = ''
            else:
                # for ordinary gzip files, write header now
                self._write_gzip_header()

        if self.mode == READ:
            # read the headers of all dictzip members, to build database of flush points
            self.memberoffset = []        # offset of member within uncompressed stream
            self.memberchlen = []         # chunk length
            self.memberflushpoints = []   # absolute flush points within this member
            self.dictzip = True
            pos = self.fileobj.tell()
            try:
                offset = 0
                while True:
                    self._iseof()
                    dictzipdata = self._read_gzip_header()
                    if dictzipdata:
                        chlen, flushpoints = dictzipdata
                        self.memberoffset.append(offset)
                        self.memberchlen.append(chlen)
                        for idx in range(len(flushpoints)):
                            flushpoints[idx] += self.fileobj.tell()
                        # keep flushpoints, including the one pointing beyond the data stream
                        self.memberflushpoints.append( flushpoints ) 
                        # point to length field at end of this member
                        # Add 4 to skip over the CRC32 field
                        # (I don't understand the "+2" -- header bytes?)
                        newpos = flushpoints[-1] + 2 + 4
                        self.fileobj.seek( newpos )
                        isize = U32(read32(self.fileobj))   # will not exceed 2 Gb for dictzip files
                        offset += isize
                    else:
                        self.dictzip = False
                        break
            except EOFError:
                pass
            self.uncompressed_length = offset
            self.fileobj.seek(pos)

    def __repr__(self):
        s = repr(self.fileobj)
        return '<gzip ' + s[1:-1] + ' ' + hex(id(self)) + '>'

    def _init_write(self, filename):
        if filename != None and filename[-3:] != '.gz' and filename[-3:] != '.dz':
            if self.dictzip:  filename = filename + '.dz'
            else:             filename = filename + '.gz'
        self.filename = filename
        self.crc = zlib.crc32("")
        self.size = 0
        self.compress = zlib.compressobj(self.compresslevel,
                                         zlib.DEFLATED,
                                         -zlib.MAX_WBITS,
                                         zlib.DEF_MEM_LEVEL,
                                         0)

    def _write_gzip_header(self, size = None):
        self.fileobj.write('\037\213')             # magic header
        self.fileobj.write('\010')                 # compression method
        flags = 0
        if self.filename:
            flags = FNAME
        if self.dictzip:
            flags |= FEXTRA
        self.fileobj.write(chr(flags))
        write32u(self.fileobj, long(time.time()))
        self.fileobj.write('\002')                 # extraflag
        self.fileobj.write('\377')                 # os (unknown)
        if self.dictzip:
            chunks = 1 + (size-1) // self.chunksize
            xlen = 10 + 2*chunks
            self.fileobj.write( chr(xlen % 256) )               # length of extra field
            self.fileobj.write( chr(xlen // 256) )
            self.fileobj.write('RA')                            # dictzip's magic word - 'R'andom 'A'ccess
            sublen = xlen-4
            self.fileobj.write( chr(sublen % 256) )             # length of subfield
            self.fileobj.write( chr(sublen // 256) )
            self.fileobj.write('\001\000')                      # dictzip header version
            self.fileobj.write( chr(self.chunksize % 256 ) )    # size of chunk
            self.fileobj.write( chr(self.chunksize // 256 ) )
            self.fileobj.write( chr(chunks % 256) )             # number of chunks
            self.fileobj.write( chr(chunks // 256) )
            self.chunktablepos = self.fileobj.tell()
            for chunk in range(chunks):
                self.fileobj.write('\000\000')                  # placeholders
        if self.filename:
            self.fileobj.write(self.filename[:-3] + '\000')

    def _init_read(self):
        self.crc = zlib.crc32("")
        self.size = 0

    def _read_gzip_extra(self):
        xlen = ord(self.fileobj.read(1))
        xlen = xlen + 256*ord(self.fileobj.read(1))
        xtra = self.fileobj.read(xlen)
        xptr = 0
        # loop over subfields
        while xptr < xlen:
            if xlen - xptr < 4: return                              # ill-formed header: magic word + subfield length required
            sublen = ord(xtra[xptr+2]) + 256*ord(xtra[xptr+3])      # subfield length
            ptr = xptr
            xptr += sublen + 4
            if xtra[ptr] != 'R' or xtra[ptr+1] != 'A': continue     # magic word for dictzip data is 'R'andom 'A'ccess
            if xtra[ptr+4] != '\001' or xtra[ptr+5] != '\000':
                raise IOError, "Unrecognized DictZip version: "+str(ord(xtra[ptr+4]) + 256*ord(xtra[ptr+5]))
            chlen = ord(xtra[ptr+6]) + 256*ord(xtra[ptr+7])         # chunk length
            chcnt = ord(xtra[ptr+8]) + 256*ord(xtra[ptr+9])         # chunk count
            if chcnt * 2 != sublen - 6:
                raise IOError, "Invalid DictZip header: wrong number of chunks:"+str(chcnt)+" expected "+str((sublen-6)//2)
            flushpoints = [0]
            for idx in range(chcnt):
                flushpoints.append( flushpoints[-1] + ord(xtra[ptr+10+2*idx]) + 256*ord(xtra[ptr+11+2*idx]) )
            return (chlen, flushpoints)                             # ignore other subfields
        if xptr != xlen:
            raise IOError, "Bad extra field in gzip header"
        return None
        
    def _read_gzip_header(self):
        magic = self.fileobj.read(2)
        if magic != '\037\213':
            raise IOError, 'Not a gzipped file'
        method = ord( self.fileobj.read(1) )
        if method != 8:
            raise IOError, 'Unknown compression method'
        flag = ord( self.fileobj.read(1) )
        # modtime = self.fileobj.read(4)
        # extraflag = self.fileobj.read(1)
        # os = self.fileobj.read(1)
        self.fileobj.read(6)
        dictzipdata = None

        if flag & FEXTRA:
            # Read the extra field
            dictzipdata = self._read_gzip_extra()
        if flag & FNAME:
            # Read and discard a null-terminated string containing the filename
            while True:
                s = self.fileobj.read(1)
                if not s or s=='\000':
                    break
        if flag & FCOMMENT:
            # Read and discard a null-terminated string containing a comment
            while True:
                s = self.fileobj.read(1)
                if not s or s=='\000':
                    break
        if flag & FHCRC:
            self.fileobj.read(2)              # Read & discard the 16-bit header CRC
        return dictzipdata

    def write(self,data):
        if self.mode != WRITE:
            import errno
            raise IOError(errno.EBADF, "write() on read-only GzipFile object")
        if self.fileobj is None:
            raise ValueError, "write() on closed GzipFile object"
        if len(data) > 0:
            self.offset += len(data)
            if self.dictzip:
                # buffered output
                self.writebuf += data
                self._flush_write_buffer(self.buffersize)
            else:
                # unbuffered output
                self._add_write_data(data)

    def _add_write_data(self,data):
        self.size = self.size + len(data)
        self.crc = zlib.crc32(data, self.crc)
        self.fileobj.write( self.compress.compress(data) )

    def _flush_write_buffer(self, buffersize):
        while len(self.writebuf) > buffersize:
            towrite = min(self.buffersize, len(self.writebuf))
            self._write_member(self.writebuf[:towrite])
            self.writebuf = self.writebuf[towrite:]

    def _write_member(self,bufdata):
        # writes complete gzip member (including dictzip table) at once
        self._write_gzip_header( len(bufdata) )
        start_of_block = self.fileobj.tell()
        block_sizes = []
        for chunkstart in range(0,len(bufdata),self.chunksize):
            # output the dictzip chunks
            self._add_write_data( bufdata[chunkstart : min(chunkstart + self.chunksize, len(bufdata))] )
            # flush the compressor and store block size
            self.fileobj.write( self.compress.flush(zlib.Z_FULL_FLUSH) )
            current_pos = self.fileobj.tell()
            block_sizes.append( current_pos - start_of_block )
            start_of_block = current_pos
        # finish this member
        self._endmember()
        # seek back and write block sizes
        current_pos = self.fileobj.tell()
        self.fileobj.seek( self.chunktablepos )
        for block_size in block_sizes:
            self.fileobj.write( chr( block_size % 256 ) )
            self.fileobj.write( chr( block_size // 256 ) )
        # return to previous position
        self.fileobj.seek( current_pos )
        # initialize - with no filename - for next member
        self._init_write(None)
    

    def read(self, size=-1, _block_read_size=None):
        if self.mode != READ:
            import errno
            raise IOError(errno.EBADF, "read() on write-only GzipFile object")

        if self.extrasize <= 0 and self.fileobj is None:
            return ''

        if not _block_read_size:
            _block_read_size = 512   # start small, in case the compression factor is high
        else:
            _block_read_size = min(_block_read_size, 2048)
        if size < 0:                 # get the whole thing
            try:
                while True:
                    self._read(_block_read_size)
                    _block_read_size = min(_block_read_size * 2, size - self.extrasize + 20)
            except EOFError:
                size = self.extrasize
        else:               # just get some more of it
            try:
                while size > self.extrasize:
                    self._read(_block_read_size)
                    _block_read_size = min(_block_read_size * 2, size - self.extrasize + 20)
            except EOFError:
                if size > self.extrasize:
                    size = self.extrasize

        chunk = self.extrabuf[:size]
        self.extrabuf = self.extrabuf[size:]
        self.extrasize = self.extrasize - size

        self.offset += size
        return chunk

    def _unread(self, buf):
        self.extrabuf = buf + self.extrabuf
        self.extrasize = len(buf) + self.extrasize
        self.offset -= len(buf)

    def _iseof(self):
        pos = self.fileobj.tell()   # Save current position
        self.fileobj.seek(0, 2)     # Seek to end of file
        if pos == self.fileobj.tell():
            raise EOFError, "Reached EOF"
        else:
            self.fileobj.seek( pos ) # Return to original position

    def _read(self, size):
        if self.fileobj is None:
            raise EOFError, "Reached EOF"

        if self._new_member:
            # If the _new_member flag is set, we have to
            # jump to the next member, if there is one.
            #
            # First, check if we're at the end of the file;
            # if so, it's time to stop; no more members to read.
            self._iseof()
            
            self._init_read()
            self._read_gzip_header()
            self.decompress = zlib.decompressobj(-zlib.MAX_WBITS)
            self._new_member = False

        # Read a chunk of data from the file
        buf = self.fileobj.read(size)

        # If the EOF has been reached, flush the decompression object
        # and mark this object as finished.

        if buf == "":
            uncompress = self.decompress.flush()
            self._read_eof()
            self._add_read_data( uncompress )
            raise EOFError, 'Reached EOF'

        uncompress = self.decompress.decompress(buf)
        self._add_read_data( uncompress )

        if self.decompress.unused_data != "":
            # Ending case: we've come to the end of a member in the file,
            # so seek back to the start of the unused data, finish up
            # this member, and read a new gzip header.
            # (The number of bytes to seek back is the length of the unused
            # data, minus 8 because _read_eof() will rewind a further 8 bytes)
            self.fileobj.seek( -len(self.decompress.unused_data)+8, 1)

            # Check the CRC and file size, and set the flag so we read
            # a new member on the next call
            self._read_eof()
            self._new_member = True

    def _add_read_data(self, data):
        self.crc = zlib.crc32(data, self.crc)
        self.extrabuf = self.extrabuf + data
        self.extrasize = self.extrasize + len(data)
        self.size = self.size + len(data)

    def _read_eof(self):
        # We've read to the end of the file, so we have to rewind in order
        # to reread the 8 bytes containing the CRC and the file size.
        # We check the that the computed CRC and size of the
        # uncompressed data matches the stored values.  Note that the size
        # stored is the true file size mod 2**32.
        if self.dictzip:
            return                          # we don't keep crc values for dictzips
        self.fileobj.seek(-8, 1)
        crc32 = read32(self.fileobj)
        isize = U32(read32(self.fileobj))   # may exceed 2GB
        if U32(crc32) != U32(self.crc):
            raise IOError, "CRC check failed"
        elif isize != LOWU32(self.size):
            raise IOError, "Incorrect length of data produced"

    def _endmember(self):
        if self.mode == WRITE:
            self.fileobj.write(self.compress.flush())  # unbuffered output
            write32(self.fileobj, self.crc)
            # self.size may exceed 2GB, or even 4GB
            write32u(self.fileobj, LOWU32(self.size))

    def close(self):
        if self.mode == WRITE:
            if self.dictzip:
                self._flush_write_buffer(0)                # writes buffers, and finishes member
            else:
                self._endmember()
            self.fileobj = None
        elif self.mode == READ:
            self.fileobj = None
        if self.myfileobj:
            self.myfileobj.close()
            self.myfileobj = None

    def __del__(self):
        try:
            if (self.myfileobj is None and
                self.fileobj is None):
                return
        except AttributeError:
            return
        self.close()

    def flush(self):
        self.fileobj.flush()

    def fileno(self):
        """Invoke the underlying file object's fileno() method.

        This will raise AttributeError if the underlying file object
        doesn't support fileno().
        """
        return self.fileobj.fileno()

    def isatty(self):
        return False

    def tell(self):
        return self.offset

    def rewind(self):
        '''Return the uncompressed stream file position indicator to the
        beginning of the file'''
        if self.mode != READ:
            raise IOError("Can't rewind in write mode")
        self.fileobj.seek(0)
        self._new_member = True
        self.extrabuf = ""
        self.extrasize = 0
        self.offset = 0

    def seek(self, offset, whence = SEEK_SET):
        if whence == SEEK_CUR:
            offset += self.offset
        if self.mode == WRITE:
            if whence == SEEK_END:
                raise IOError('SEEK_END not supported in write mode')
            if offset < self.offset:
                raise IOError('Negative seek in write mode')
            count = offset - self.offset
            for i in range(count // 1024):
                self.write(1024 * '\0')
            self.write((count % 1024) * '\0')
        elif self.mode == READ:
            readahead = None
            if self.dictzip:
                if whence == SEEK_END:
                    offset += self.uncompressed_length
                count = offset - self.offset
                if count >= 0 and count < 32768:
                    # small forward seeks
                    pass
                else:
                    # use table of flush points
                    member = max(0, bisect.bisect_right(self.memberoffset, offset) - 1)
                    memberoffset = self.memberoffset[member]                           # Start of member's uncompressed stream
                    chlen = self.memberchlen[member]                                   # Chunk length
                    flushpoints = self.memberflushpoints[member]
                    idx = min((offset - memberoffset) // chlen, len(flushpoints)-2 )   # Exclude the last entry, pointing beyond data stream
                    readahead = flushpoints[min(idx+2,len(flushpoints)-1)] - flushpoints[idx]
                    self.fileobj.seek( flushpoints[idx] )                              # Seek to start of chunk
                    self.offset = memberoffset + idx * chlen                           # Offset of chunk in uncompressed stream
                    self.size = idx * chlen                                            # Size is relative to member (ignored)
                    self.extrabuf = ""
                    self.extrasize = 0
                    self.crc = zlib.crc32("")                                          # CRC is invalid (ignored)
                    self.decompress = zlib.decompressobj(-zlib.MAX_WBITS)
                    self._new_member = False                                           # Reset possible EOF
                    count = offset - self.offset
            else:
                if whence == SEEK_END:
                    raise IOError("SEEK_END only supported on gzip files with a random-access header")
                count = offset - self.offset
                if count < 0:
                    # for backward seek, rewind and do positive seek. 
                    # If self.buffersize is set, take this as a hint that a dictzip file
                    # was expected, and refuse to do the (very inefficient) seek
                    if self.buffersize:
                        raise IOError("Negative seek on non-dictzip gzip files attempted")
                    self.rewind()
                    count = offset
            # read away unwanted bytes.  A 32K block size appears to be most efficient
            for i in range(count // 32768):
                self.read(32768, _block_read_size = readahead)
            self.read(count % 32768, _block_read_size = readahead)

    def readline(self, size=-1):
        if size < 0: size = sys.maxint
        bufs = []
        readsize = min(100, size)    # Read from the file in small chunks
        while True:
            if size == 0:
                return "".join(bufs) # Return resulting line

            c = self.read(readsize)
            i = c.find('\n')
            if size is not None:
                # We set i=size to break out of the loop under two
                # conditions: 1) there's no newline, and the chunk is
                # larger than size, or 2) there is a newline, but the
                # resulting line would be longer than 'size'.
                if i==-1 and len(c) > size: i=size-1
                elif size <= i: i = size -1

            if i >= 0 or c == '':
                bufs.append(c[:i+1])    # Add portion of last chunk
                self._unread(c[i+1:])   # Push back rest of chunk
                return ''.join(bufs)    # Return resulting line

            # Append chunk to list, decrease 'size',
            bufs.append(c)
            size = size - len(c)
            readsize = min(size, readsize * 2)

    def readlines(self, sizehint=0):
        # Negative numbers result in reading all the lines
        if sizehint <= 0:
            sizehint = sys.maxint
        L = []
        while sizehint > 0:
            line = self.readline()
            if line == "":
                break
            L.append(line)
            sizehint = sizehint - len(line)

        return L

    def writelines(self, L):
        for line in L:
            self.write(line)

    def __iter__(self):
        return self

    def next(self):
        line = self.readline()
        if line:
            return line
        else:
            raise StopIteration


def _test():
    # Act like gzip; with -d, act like gunzip; with -D, act like dictzip
    # The input file is not deleted, however, nor are any other gzip
    # options or features supported.
    args = sys.argv[1:]
    decompress = args and args[0] == "-d"
    dictzip = args and args[0] == "-D"
    if decompress or dictzip:
        args = args[1:]
    if not args:
        args = ["-"]
    for arg in args:
        if decompress:
            if arg == "-":
                f = GzipFile(filename="", mode="rb", fileobj=sys.stdin)
                g = sys.stdout
            else:
                if arg[-3:] != ".gz" and arg[-3:] != ".dz":
                    print "filename doesn't end in .gz or .dz:", repr(arg)
                    continue
                f = open(arg, "rb")
                g = __builtin__.open(arg[:-3], "wb")
        else:
            if dictzip:
                buffersize = 1000000
                ext = ".dz"
            else:
                buffersize = None
                ext = ".gz"
            if arg == "-":
                f = sys.stdin
                g = GzipFile(filename="", mode="wb", fileobj=sys.stdout, chunksize = 1000, buffersize = buffersize)
            else:
                f = __builtin__.open(arg, "rb")
                g = open(arg + ext, "wb", chunksize = 1000, buffersize = buffersize)
        blocksize = 1024
        if False:
            while True:
                chunk = f.read(blocksize)
                if not chunk:
                    break
                g.write(chunk)
        else:
            # test the random access code
            ptr = 0
            while True:
                f.seek(0)
                f.seek(ptr)
                chunk = f.read(blocksize)
                if not chunk:
                    break
                g.write(chunk)
                ptr += blocksize
        if g is not sys.stdout:
            g.close()
        if f is not sys.stdin:
            f.close()

if __name__ == '__main__':
    _test()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
psl2predictions.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python psl2predictions.py --help

Type::

   python psl2predictions.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import getopt

USAGE="""python %s [OPTIONS] < psl > predictions

Convert BLAT output to predictions.

Version: $Id: psl2predictions.py 14 2005-08-09 15:24:07Z andreas $

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-t, --trans                     input is translated DNA
""" % sys.argv[0]


param_long_options=["verbose=", "help", "trans", "version"]
param_short_options="v:ht"

param_trans = None

import CGAT.Experiment as E
import CGAT.PredictionParser as PredictionParser

if __name__ == "__main__":

    try:
        optlist, args = getopt.getopt(sys.argv[1:], param_short_options, param_long_options)
    except getopt.error, msg:
        print USAGE, msg
        sys.exit(2)

    for o,a in optlist:
        if o in ( "-v", "--verbose" ):
            param_loglevel = int(a)
        elif o in ( "--version", ):
            print "version="
            sys.exit(0)
        elif o in ( "-h", "--help" ):
            print USAGE
            sys.exit(0)
        elif o in ( "-t", "--trans"):
            param_trans = 1

    print E.GetHeader()
    print E.GetParams()

    if param_trans:
        parser = PredictionParser.PredictionParserBlatTrans()
    else:
        parser = PredictionParser.PredictionParserBlatCDNA()


    nmatches = 1
    for line in sys.stdin:
        if line[0] == "#": continue
        if not re.match("^[0-9]", line): continue

        try:
            entries = parser.Parse((line,))
        except PredictionParser.AlignmentError, e:
            print "# %s" % str(e)
            print "#", line[:-1]
            sys.exit(1)
        
        for entry in entries:
            entry.mPredictionId = nmatches
            nmatches += 1
            
        print str(entries)

    print E.GetFooter()

    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: bed2psl.py 2899 2010-04-13 14:37:37Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
bed2psl.py - convert a bed file to a psl file
=============================================

:Author: Andreas Heger
:Release: $Id: bed2psl.py 2899 2010-04-13 14:37:37Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse

import CGAT.Experiment as E

import CGAT.Blat as Blat
import CGAT.Bed as Bed
import CGAT.IndexedFasta as IndexedFasta
            
def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: bed2psl.py 2899 2010-04-13 14:37:37Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-q", "--query", dest="query", type="string",
                      help="sequence to use for query [default=%default]."  )

    parser.add_option("-t", "--target", dest="target", type="string",
                      help="sequence to use for target [default=%default]."  )

    parser.add_option("-g", "--genome-file", dest="genome_file", type="string",
                      help="filename with genome."  )

    parser.set_defaults(
        genome_file = None,
        query = None,
        target = None,
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    if options.genome_file:
        fasta = IndexedFasta.IndexedFasta( options.genome_file )
    else:
        fasta = None

    psl = Blat.Match()

    for bed in Bed.iterator(options.stdin):

        ninput += 1

        start, end = bed.start, bed.end

        if "blockSizes" in bed:
            psl.mQueryId = bed["name"]
            blocksizes = [ int(x) for x in bed["blockSizes"].split(",")[:-1]]
            sbjctblockstarts = [ int(x) + start for x in bed["blockStarts"].split(",")[:-1]]
            strand = bed["strand"]
        else: 
            psl.mQueryId = "%i" % ninput
            blocksizes = [end - start ]
            sbjctblockstarts = [start,]

            strand = "+"
        
        psl.mSbjctId = bed.contig
        psl.mSbjctFrom, psl.mSbjctTo = start, end
        psl.mQueryFrom, psl.mQueryTo = 0, end - start

        psl.mBlockSizes = blocksizes
        psl.mNBlocks = len(blocksizes)
        psl.strand = strand
        q, qp = [], 0
        for x in blocksizes:
            q.append( qp )
            qp += x 

        psl.mQueryBlockStarts = q
        psl.mSbjctBlockStarts = sbjctblockstarts
        psl.mQueryLength = sum( psl.mBlockSizes )
        if fasta:
            psl.mSbjctLength = fasta.getLength( bed.contig )

        options.stdout.write( "%s\n" % str(psl) )
        noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff_compare.py - compare two gene sets
======================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script benchmarks two gene predictions. Predictions are given as gff files.

This script is transcript aware but not gene aware, i.e. if there are several
transcripts for a gene, overcounting happens.

Usage
-----

Example::

   python gff_compare.py --help

Type::

   python gff_compare.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse

USAGE="""python %s [OPTIONS] target reference

""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.PredictionParser as PredictionParser
import numpy

##------------------------------------------------------------------------
def Old():
    
    ref_nucleotides    = numpy.zeros( max_end - min_start, numpy.int0 )
    target_nucleotides = numpy.zeros( max_end - min_start, numpy.int0 )

    # set range overlapping by reference
    for x in targets:
        target_nucleotides[x.start - min_start:x.end - min_start] = 1

    for x in references:
        ref_nucleotides[x.start - min_start:x.end - min_start] = 1

    not_ref = numpy.logical_not(ref_nucleotides)
    not_target = numpy.logical_not(target_nucleotides)
    
    v = numpy.logical_and( ref_nucleotides, target_nucleotides) 
    tp = numpy.dot( v, v )

    v = numpy.logical_and( ref_nucleotides, not_target )
    fn = numpy.dot( v, v )    
    
    v = numpy.logical_and( not_ref, target_nucleotides) 
    fp = numpy.dot( v, v )

    v = numpy.logical_and( not_ref, not_target)
    tn = numpy.dot( v, v )

##------------------------------------------------------------------------
def GetCounts( nucleotides, val ):
    """count number of occurances of val in array.
    
    can't find a way without running into memory troubles.
    """
    t = 0
    for x in nucleotides:
        if x == val: t += 1
    return t

##------------------------------------------------------------------------
def AnalyseOverlaps( references, targets ):
    """calculate nucleotide overlap.

    All references and targets should be on the same chromsome.
    Name and strand are not checked.
    """

    if len(references) == 0 or len(targets) == 0:
        return( (0,0,0,0) )
    
    targets_start = min( map(lambda x: x.start, targets) )
    targets_end   = max( map(lambda x: x.end, targets) )

    refs_start = min( map(lambda x: x.start, references) )
    refs_end   = max( map(lambda x: x.end, references) )
    
    min_start = min( targets_start, refs_start )
    max_end   = max( targets_end, refs_end)

    nucleotides    = numpy.zeros( max_end - min_start, numpy.int0 )
    
    # set ranges: bit 0: target, bit 1: reference
    for x in targets[:100]:
        nucleotides[x.start - min_start:x.end - min_start] = 1

    for x in references[:100]:
        nucleotides[x.start - min_start:x.end - min_start] = \
                             numpy.array( [2] * (x.end - x.start), numpy.int0 )

    tp = GetCounts( nucleotides, 3 )
    fp = GetCounts( nucleotides, 1 )
    tn = GetCounts( nucleotides, 0 )
    fn = GetCounts( nucleotides, 2 )
    
    return (tp, fp, tn, fn )

##------------------------------------------------------------------------
def CalculateSpecificitySensitivity( tp, fp, tn, fn ):

    if fp + tp == 0:
        spec = 1.0
    else:
        spec = float(tp) / (fp + tp)

    if tp + fn == 0:
        sens = 0.0
    else:
        sens = float(tp) / (tp + fn)
        
    return spec, sens

##------------------------------------------------------------------------
def CalculateCorrelationCoefficient( tp, fp, tn, fn ):

    x = ( ( tp + fn ) * ( tn + fp ) * ( tp + fp ) * ( tn + fn ) )
    if x == 0:
        return 0.0
    else:
        return (tp * tn - fn * fp ) / x
               

##------------------------------------------------------------------------
def GetFirstOverlaps( gffs, index ):
    """get next overlapping gffs from array starting at index.
    """
    
    this_name = gffs[index].mName
    this_strand = gffs[index].strand
    l = len(gffs)
    min_start = gffs[index].start
    max_end = gffs[index].end

    overlaps = [gffs[index]]
    index += 1
    
    while index < l:
        if this_name != gffs[index].mName or \
           this_strand != gffs[index].strand or\
           max_end < gffs[index].start:
            return overlaps, index, min_start, max_end

        max_end = max( max_end, gffs[index].end )
        overlaps.append( gffs[index] )
        index += 1
        
    return overlaps, index, min_start, max_end        

##------------------------------------------------------------------------
def CountMatchesPerGene( gffs,
                         rx_gene,
                         rx_other,
                         write = (),
                         outfile = sys.stdout ):
    """use status information in gffs to check for completely matches/extra genes.
    """

    # map of gene to matching exons
    # match contains tuple of:
    # nexons, nextra, complete matches to other_gene, partial matches to other gene
    map_gene2matches = {}
    for x in gffs:

        try:
            gene = rx_gene.search( x.mInfo ).groups()[0]
        except AttributeError:
            print "# ERROR: could not find gene identifier in %s" % x.mInfo
            sys.stdout.flush()
            sys.exit(1)
         
        # total, extra, matches(per other gene), partial(per other gene)
        if gene not in map_gene2matches:
            map_gene2matches[gene] = [0, 0, {}, {}] 

        info = map_gene2matches[gene]
        info[0] += 1

        try:
            if x.mStatus == "extra":
                info[1] += 1
            elif x.mStatus in ("match", "partial"):

                for y in x.mMatches:
                    other_gene = rx_other.search( y.mInfo ).groups()[0] 
                    if other_gene not in info[2]: info[2][other_gene] = 0
                    if other_gene not in info[3]: info[3][other_gene] = 0

                    if GTF.Identity( x, y, max_slippage=options.max_exon_slippage ):
                        info[2][other_gene] += 1
                        info[3][other_gene] += 1                    
                    elif GTF.HalfIdentity( x, y, max_slippage=options.max_exon_slippage ):
                        info[3][other_gene] += 1
        except AttributeError:
            print "Programming ERROR: no mStatus for ", str(x)
            
    total_extra = 0
    total_match = 0
    total_partial_match = 0
    total = len(map_gene2matches)
    for gene, info in map_gene2matches.items():

        l, nextra, matches, partials = info
        
        if nextra == l:
            total_extra += 1
            if "missed" in write:
                outfile.write( "missed\t%s\n" % (gene,))                                        
        else:
            for x, y in matches.items():
                if y == l:
                    total_match += 1
                    if "match" in write:
                        outfile.write( "match\t%s\t%s\n" % (gene, x))                        
                    break
                        
            else:
                for x,y in partials.items():            
                    if y == l:
                        total_partial_match += 1
                        if "partial" in write:
                            outfile.write( "partial\t%s\t%s\n" % (gene, x))
                        break

    return total, total_match, total_partial_match, total_extra

##------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff_compare.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-f", "--write-full", dest="write_full",
                      help="write full gff entries.", action="store_true"  )
    parser.add_option("-e", "--write-matched-exons", dest="write_matched_exons", 
                      help="write matched exons.", action="store_true"  )
    parser.add_option("-o", "--write-missed-exons", dest="write_missed_exons", action="store_true",
                      help="write missed exons."  )
    parser.add_option("-g", "--write-missed-genes", dest="write_missed_genes", action="store_true",
                      help="write missed genes."  )
    parser.add_option("-r", "--regex-reference", dest="regex_reference", type="string",
                      help="regular expression mapping exon to transcript in reference."  )
    parser.add_option("-t", "--regex-target", dest="regex_target", type="string",
                      help="regular expression mapping exon to transcript in target."  )
    parser.add_option("--no-nucleotides", dest="do_nucleotides", action="store_false",
                      help="skip nucleotide benchmark."  )
    parser.add_option("--no-exons", dest="do_exons", action="store_false",
                      help="skip exon benchmark."  )
    parser.add_option("--no-genes", dest="do_genes", action="store_false",
                      help="skip gene benchmark."  )
    parser.add_option("--out-filename-pattern", dest="outfile_pattern", type="string",
                      help="output filename pattern for extra info (%s will be substituted with reference,target)."  )
    
    parser.set_defaults(
        remove_redundancy = False,
        max_exon_slippage = 9,
        write_missed_exons = False,
        write_matched_exons = False,
        write_missed_genes = False,
        write_wrong_exons = False,
        write_wrong_genes = False,
        do_nucleotides = True,
        do_exons = True,
        do_genes = True,
        regex_reference = None,
        regex_target = None,
        outfile_pattern = "%s.info",
        )

    (options, args) = E.Start( parser )

    if len(args) != 2:
        print USAGE
        print "two arguments required"
        sys.exit(1)

    input_filename_target, input_filename_reference = args

    if options.loglevel >= 1:
        print "# target entries from %s" % input_filename_target
        print "# reading target entries ...",
        sys.stdout.flush()
        
    gff_targets = GTF.readFromFile( open( input_filename_target, "r" ) )
    
    if options.loglevel >= 1:
        print "finished: %i" % (len(gff_targets))
        sys.stdout.flush()

    if options.loglevel >= 1:
        print "# reference entries from %s" % input_filename_reference
        print "# reading reference entries ...",
        sys.stdout.flush()    

    gff_references = GTF.readFromFile( open( input_filename_reference, "r" ) )

    if options.loglevel >= 1:
        print "finished: %i" % (len(gff_references))
        sys.stdout.flush()

    if options.remove_redundancy:
        gff_targets = GTF.CombineOverlaps( gff_targets )
        gff_references = GTF.CombineOverlaps( gff_references )    

        if options.loglevel >= 1:    
            print "# after filtering: targets=%i, references=%i" % (len(gff_targets), len(gff_references))

    #################################################################################
    ## sort exons
    if options.loglevel >= 1:
        print "# sorting exons ...",
        sys.stdout.flush()
        
    gff_targets.sort( lambda x,y: cmp( (x.mName, x.strand, x.start, x.end),
                                       (y.mName, y.strand, y.start, y.end) ) )
    

    gff_references.sort( lambda x,y: cmp( (x.mName, x.strand, x.start, x.end),
                                          (y.mName, y.strand, y.start, y.end) ) )
    

    ntargets = len(gff_targets)
    nreferences = len(gff_references)

    if options.loglevel >= 1:
        print "finished"
        sys.stdout.flush()
        
    #################################################################################
    # get nucleotide level accuracy
    # process each fragment separately
    if options.do_nucleotides:
        print """##############################################################################
# nucleotide accuracy
#
# TP: true postives  : nucleotides to be predicted coding in both target and reference.
# FN: true negatives : nucleotides to be predicted non-coding in both target and reference.
# FP: false positives: nucleotides to be predicted non-coding in target but not in reference.
# FN: false negatives: nucleotides to be predicted non-coding in reference but not in target.
#
# Sensitivity: TP / TP + FN = correct exons / actual exons
# Specificity: TP / TP + FP
# CC: correlation coefficient:  (tp*tn-fn*fp)/((tp+fn)*(tn+fp)*(tp+fp)*(tn+fn))"""
    
        headers = ("contig", "strand", "tp", "fp", "tn", "fn", "sp", "sn", "cc" )
    
        print "\t".join(headers)
        
        first_r, first_t = 0, 0
        r, t = 0, 0

        ttp, tfp, ttn, tfn = 0, 0, 0, 0

        ## this only works, if all contigs in reference are present in target.
        while r < nreferences and t < ntargets:

            this_name = gff_references[r].mName
            this_strand = gff_references[r].strand

            ## get all in references
            while r < nreferences and \
                      gff_references[r].mName == this_name and \
                      gff_references[r].strand == this_strand:
                r+= 1

            ## skip over extra contigs in target
            while t < ntargets and \
                      (gff_targets[t].mName != this_name or \
                       gff_targets[t].strand != this_strand):
                t+= 1
            first_t = t

            ## get all in targets
            while t < ntargets and \
                      gff_targets[t].mName == this_name and \
                      gff_targets[t].strand == this_strand:                  
                t+= 1
                
            tp, fp, tn, fn =  AnalyseOverlaps( gff_references[first_r:r],
                                               gff_targets[first_t:t] )

            spec, sens = CalculateSpecificitySensitivity( tp, fp, tn, fn )
            cc = CalculateCorrelationCoefficient( tp, fp, tn, fn )
            print "%s\t%s\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f" % (this_name, this_strand, tp, fp, tn, fn, spec, sens, cc )

            ttp += tp
            tfp += fp
            ttn += tn
            tfn += fn
            first_r, first_t = r, t

        spec, sens = CalculateSpecificitySensitivity( ttp, tfp, ttn, tfn )
        cc = CalculateCorrelationCoefficient( ttp, tfp, ttn, tfn )
        print "%s\t%s\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f" % ("all", "all", ttp, tfp, ttn, tfn, spec, sens, cc )

        sys.stdout.flush()
        
    #################################################################################
    if options.do_exons or options.do_genes:

        print """##############################################################################
# exon accuracy (see Burset & Guigo (1996))
#
# TP: true postives  : exons matching both exon boundaries exactly (a deviation of
#                         maximal %i nucleotides is allowed).
# FN: true negatives : not counted.
# FP: false positives: exons in target not overlapping with in reference.
# FN: false negatives: exons overlapping, but with deviant boundaries and exons
#                         in reference not overlapping with any target exon.
#
# ME: missed exons: exons in reference not overlapping with exons in target
#                 (add to false negative count).
# WE: wrong exons: exons in target not overlapping with exons in reference
#                 (add to false positive count).
#
# Sensitivity: TP / (TP + FN) = correct exons / actual exons
# Specificity: TP / (TP + FP) = correct exons / predicted exons
# CC: (sensitivity + specificity) / 2
#
# Values are calculated where both exon boundaries of a target exon have to be
# correct (category "full") and where only one exon boundary has to be correct (catgory "half").""" %\
        (options.max_exon_slippage)

        headers = ("category", "contig", "strand", "tp", "fp", "tn", "fn", "sp", "sn", "cc", "me", "we", "me", "we" )

        print "\t".join(headers)

        r, t = 0, 0
        next_r, next_t = r, t

        # strict false positves/negatives
        tp, fp, tn, fn = 0, 0, 0, 0
        ttp, tfp, ttn, tfn = 0, 0, 0, 0
        # partial false positives/negatives
        ptp, pfp, ptn, pfn = 0, 0, 0, 0
        tptp, tpfp, tptn, tpfn = 0, 0, 0, 0

        # missed and wrong exons
        missed_exons, wrong_exons = 0, 0
        tmissed_exons, twrong_exons = 0, 0    

        ## Flag set, if partial overlap in previous pair
        last_partial_overlap = False
        ## Flag set, if partial overlap and reference was last increased
        last_increased_ref = False

        while r < nreferences and t < ntargets:

            this_name = gff_references[r].mName
            this_strand = gff_references[r].strand

            ## get overlap segments
            if next_r == r:
                ref_overlaps, next_r, ref_start, ref_end = GetFirstOverlaps( gff_references, r )
            if next_t == t:
                target_overlaps, next_t, target_start, target_end = GetFirstOverlaps( gff_targets, t )        

            if options.loglevel >= 3:
                print "########################################################"
                for x in ref_overlaps:
                    print "#", str(x)
                for x in target_overlaps:            
                    print "#", str(x)

            do_summary = False
            ## check strand switch in reference
            if next_r < nreferences and \
                   (this_name != gff_references[next_r].mName or \
                    this_strand != gff_references[next_r].strand):
                if options.loglevel >= 3:
                    print "# target advance"
                do_summary = True

                last_increased_ref = False
                last_partial_overlap = False
                    
                ## advance in target until next name is found
                next_name = gff_references[next_r].mName
                next_strand = gff_references[next_r].strand            
                while next_t < ntargets and \
                          next_name != gff_targets[next_t].mName or \
                          next_strand != gff_targets[next_t].strand:
                    fp += 1
                    pfp += 1
                    target_overlaps, next_t, target_start, target_end = GetFirstOverlaps( gff_targets, next_t )

                for x in gff_targets[t:next_t]: x.mStatus = "extra"
                for x in gff_references[r:next_r]: x.mStatus = "extra"                
                
                r, t = next_r, next_t
            ## check strand switch in target
            elif next_t < ntargets and \
                     (this_name != gff_targets[next_t].mName or \
                      this_strand != gff_targets[next_t].strand):
                    ## advance in reference until next name is found
                if options.loglevel >= 3:
                    print "# reference advance"
                do_summary = True

                last_increased_ref = False
                last_partial_overlap = False
                
                next_name = gff_targets[next_t].mName
                next_strand = gff_targets[next_t].strand            
                while next_r < nreferences and \
                          next_name != gff_references[next_r].mName or \
                          next_strand != gff_references[next_r].strand:
                    fn += 1
                    pfn += 1
                    reference_overlaps, next_r, references_start, references_end = GetFirstOverlaps( gff_references, next_r )

                for x in gff_targets[t:next_t]: x.mStatus = "extra"                
                for x in gff_references[r:next_r]: x.mStatus = "extra"

                r, t = next_r, next_t
            ## otherwise                
            else:

                ref_status, target_status = None, None
                
                if options.loglevel >= 3:
                    print "# same chromosome"
                        
                ## overlap between segments            
                if min(ref_end, target_end) - max(ref_start, target_start) > 0:

                    # clear flags
                    last_increased_ref = False
                    last_partial_overlap = False
                    found = False
                    
                    for rr in ref_overlaps:
                        xfound = False
                        for tt in target_overlaps:
                            if GTF.Identity( rr, tt, max_slippage=options.max_exon_slippage ):
                                xfound = True
                                break
                        if xfound:
                            found = True
                            break
                        
                    if found:
                        ref_status = "match"
                        target_status = "match"
                        tp += 1
                        ptp += 1
                        if options.write_matched_exons:
                            print "############# matching exons ###########################" 
                            for x in ref_overlaps:
                                print "#", str(x)
                            for x in target_overlaps:            
                                print "#", str(x)
                    else:
                        fn += 1

                        ## check for one-sided matches
                        for rr in ref_overlaps:
                            xfound = False
                            for tt in target_overlaps:
                                if GTF.HalfIdentity( rr, tt, max_slippage=options.max_exon_slippage ):
                                    xfound = True
                                    break
                            if xfound:
                                found = True
                                break

                        if found:
                            ptp += 1
                            code = "partial"
                            ref_status = "partial"
                            target_status = "partial"
                        else:
                            pfn += 1
                            code = "complete"
                            ref_status = "mismatch"
                            target_status = "mismatch"
                            
                        if options.write_missed_exons:
                            print "############# %s non-overlapping exons ###########################" % code
                            for x in ref_overlaps:
                                print "#", str(x)
                            for x in target_overlaps:            
                                print "#", str(x)
                                
                    #################################################################
                    ## r, t = next_r, next_t                                                        
                    if ref_end == target_end:
                        r, t = next_r, next_t
                    elif ref_end < target_end:
                        r = next_r
                        last_increased_ref = True
                        last_partial_overlap = True
                    else:
                        t = next_t
                        last_increased_ref = False
                        last_partial_overlap = True
                        
                ## non-overlap between segments                
                else:

                    if ref_end < target_start:
                        
                        # for non-overlap, check whether there was partial overlap before
                        # and reference was not increased.
                        # if there was, just increment reference, but do not count.

                        if not (last_partial_overlap and not last_increased_ref):
                        
                            if options.write_missed_exons:
                                print "############# missed exon ###########################" 
                                for x in ref_overlaps:
                                    print "#", str(x)
                            missed_exons += 1
                            fn += 1
                            pfn += 1
                            ref_status = "extra"
                            
                        r = next_r
                        
                    else:
                        
                        # for non-overlap, check whether there was partial overlap before
                        # and target was not increased.
                        # if there was, just increment target, but do not count.

                        if not (last_partial_overlap and last_increased_ref):
                            if options.write_wrong_exons:
                                print "############# wrong exon ###########################" 
                                for x in target_overlaps:
                                    print "#", str(x)
                            
                            wrong_exons += 1
                            fp += 1
                            pfp += 1
                            target_status = "extra"
                            
                        t = next_t
                        
                    last_partial_overlap = False

                if options.loglevel >= 3:
                    print "# ref_status=%s, target_status=%s" % (ref_status, target_status)
                    
                if ref_status:
                    for rr in ref_overlaps:
                        rr.mStatus = ref_status
                        
                    if ref_status in ("match", "partial") and options.do_genes:
                        for rr in ref_overlaps:                            
                            rr.mMatches = target_overlaps

                if target_status:
                    for tt in target_overlaps:
                        tt.mStatus = target_status
                    
                    if target_status in ("match", "partial") and options.do_genes:
                        for tt in target_overlaps:
                            tt.mMatches = ref_overlaps

            if do_summary or r >= nreferences or t >= ntargets:
                ttp += tp
                tfp += fp
                ttn += tn
                tfn += fn

                tptp += ptp
                tpfp += pfp
                tptn += ptn
                tpfn += pfn

                tmissed_exons += missed_exons
                twrong_exons += wrong_exons

                if tp + fn != 0:
                    pmissed_exons = "%5.2f" % (float(missed_exons) / (tp + fn))
                else:
                    pmissed_exons = "0"

                if tp + fp != 0:
                    pwrong_exons = "%5.2f" % (float(wrong_exons) / (tp + fp))
                else:
                    pwrong_exons = "na"
                    
                spec, sens = CalculateSpecificitySensitivity( tp, fp, tn, fn )
                cc = (spec + sens) / 2.0
                print "full\t%s\t%s\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f\t%i\t%i\t%s\t%s" % \
                      (this_name, this_strand,
                       tp, fp, tn, fn,
                       spec, sens, cc, 
                       missed_exons, wrong_exons,
                       pmissed_exons, pwrong_exons )

                spec, sens = CalculateSpecificitySensitivity( ptp, pfp, ptn, pfn )
                cc = (spec + sens) / 2.0
                print "half\t%s\t%s\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f\t%i\t%i\t%s\t%s" %\
                      (this_name, this_strand,
                       ptp, pfp, ptn, pfn,
                       spec, sens, cc,
                       missed_exons, wrong_exons,
                       pmissed_exons, pwrong_exons )

                tp, fp, tn, fn = 0, 0, 0, 0
                ptp, pfp, ptn, pfn = 0, 0, 0, 0
                missed_exons, wrong_exons = 0, 0

        if t < ntargets:
            for x in gff_targets[t:ntargets]: x.mStatus = "extra"
        if r < nreferences:            
            for x in gff_references[r:nreferences]: x.mStatus = "extra"

        spec, sens = CalculateSpecificitySensitivity( ttp, tfp, ttn, tfn )
        cc = (spec + sens) / 2.0
        print "full\t%s\t%s\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f\t%i\t%i\t%5.2f\t%5.2f" % \
              ("all", "all", ttp, tfp, ttn, tfn,
               spec, sens, cc,
               tmissed_exons, twrong_exons,
               float(tmissed_exons) / (ttp + tfn),
               float(twrong_exons) / (ttp + tfp ) )

        spec, sens = CalculateSpecificitySensitivity( tptp, tpfp, tptn, tpfn )
        cc = (spec + sens) / 2.0
        print "half\t%s\t%s\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f\t%i\t%i\t%5.2f\t%5.2f" %\
              ("all", "all", tptp, tpfp, tptn, tpfn,
               spec, sens, cc,
               tmissed_exons, twrong_exons,
               float(tmissed_exons) / (ttp + tfn),
               float(twrong_exons) / (ttp + tfp ) )             

    if options.do_genes and \
           options.regex_reference and \
           options.regex_target:

        print """##############################################################################
# gene accuracy (see Burset & Guigo (1996))
#
# TP: true postives  : genes with exons matching both exon boundaries exactly (a deviation of
#                         maximal %i nucleotides is allowed).
# FN: true negatives : not counted.
# FP: false positives: genes in target not overlapping with in reference.
# FN: false negatives: genes overlapping, but with deviant exon boundaries and genes
#                         in reference not overlapping with any target exon.
#
# MG: missed genes: exons in reference not overlapping with exons in target
#                 (add to false negative count).
# WG: wrong genes: exons in target not overlapping with exons in reference
#                 (add to false positive count).
#
# Sensitivity: TP / (TP + FN) = correct genes / actual genes
# Specificity: TP / (TP + FP) = correct genes / predicted genes
# CC: (sensitivity + specificity) / 2
#
# Values are calculated where both exon boundaries of a target exon have to be
# correct (category "full") and where only one exon boundary has to be correct (catgory "half").""" %\
        (options.max_exon_slippage)

        out_options = []
        if options.write_missed_genes:
            out_options.append("missed")

        if options.loglevel >= 2:
            print "# counting matches for reference."
            sys.stdout.flush()
            
        (ref_total, ref_match, ref_partial, ref_extra) =\
                    CountMatchesPerGene( gff_references,
                                         re.compile(options.regex_reference),
                                         re.compile(options.regex_target),
                                         write = out_options,
                                         outfile = open(options.outfile_pattern % "reference", "w") )                                         

        if options.loglevel >= 2:
            print "# counting matches for target."
            sys.stdout.flush()
        
        (target_total, target_match, target_partial, target_extra) =\
                       CountMatchesPerGene( gff_targets,
                                            re.compile(options.regex_target),
                                            re.compile(options.regex_reference),
                                            write = out_options,
                                            outfile = open(options.outfile_pattern % "target", "w") )        

        if options.loglevel >= 1:
            print "# reference: genes=%6i, matches=%6i, partial=%6i, extra=%6i" % \
                  (ref_total, ref_match, ref_partial, ref_extra)
            print "# target   : genes=%6i, matches=%6i, partial=%6i, extra=%6i" % \
                  (target_total, target_match, target_partial, target_extra)            
        
        headers = ("category", "tp", "fp", "tn", "fn", "sp", "sn", "cc", "mg", "wg", "mg", "wg" )        
        print "\t".join(headers)

        tp = ref_match
        fp = target_extra
        tn = 0
        fn = ref_total - ref_match
        wrong_genes = target_extra
        missed_genes = ref_extra 

        spec, sens = CalculateSpecificitySensitivity( tp, fp, tn, fn )
        cc = (spec + sens) / 2.0

        if tp + fp == 0: fp = nreferences
        
        print "full\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f\t%i\t%i\t%5.2f\t%5.2f" % \
              (tp, fp, tn, fn,
               spec, sens, cc,
               missed_genes, wrong_genes,
               float(missed_genes) / (tp + fn),
               float(wrong_genes) / (tp + fp) )

        tp = ref_match + ref_partial
        fp = target_extra
        tn = 0
        fn = ref_total - ref_match - ref_partial
        wrong_genes = target_extra
        missed_genes = ref_extra 
        
        spec, sens = CalculateSpecificitySensitivity( tp, fp, tn, fn )
        cc = (spec + sens) / 2.0
        print "half\t%i\t%i\t%i\t%i\t%5.2f\t%5.2f\t%5.2f\t%i\t%i\t%5.2f\t%5.2f" % \
              (tp, fp, tn, fn,
               spec, sens, cc,
               missed_genes, wrong_genes,
               float(missed_genes) / (tp + fn),
               float(wrong_genes) / (tp + fp) )

    E.Stop()    





################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
GDLDraw.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Code
----

'''
class Node:

    def __init__(self, title,
                 label = None,
                 colour = None,
                 info1 = None,
                 info2 = None,
                 vertical_order = None,
                 horizontal_order = None,
                 shape = None,
                 height = None,
                 width = None,
                 location = None):

        self.mLabel = label
        self.mTitle = title
        self.mInfo1 = info1
        self.mInfo2 = info2
        self.mVerticalOrder = vertical_order
        self.mHorizontalOrder = horizontal_order
        self.mColour = colour
        self.mShape = shape
        self.mHeight = height
        self.mWidth = width
        self.mLocation   = location

    def Write( self, outfile ):
        """write node to outputfile."""

        statement = ' title: "%s" ' % self.mTitle
        
        if self.mLabel != None:
            statement += ' label: "%s" ' % self.mLabel

        if self.mShape: statement += ' shape: %s ' % self.mShape
        if self.mInfo1: statement += ' info1: "%s" ' % self.mInfo1
        if self.mInfo2: statement += ' info2: "%s" ' % self.mInfo2        
        if self.mVerticalOrder: statement += ' vertical_order: %i ' % self.mVerticalOrder
        if self.mHorizontalOrder: statement += ' horizontal_order: %i ' % self.mHorizontalOrder        
        if self.mColour: statement += ' color: %s ' % self.mColour    
        if self.mHeight: statement += ' height: %i ' % self.mHeight
        if self.mWidth: statement += ' width: %i ' % self.mWidth        
        if self.mLocation: statement += ' loc: { x: %i y: %i } ' % self.mLocation
        
        outfile.write( '\tnode: { ' + statement + '}\n')

##---------------------------------------------------------------------------
class Edge:
    
    def __init__( self, source_name, target_name,
                  colour = None,
                  line_style = None,
                  type = 'edge',
                  priority = None,
                  edge_class = None):
        
        self.sourceName = source_name
        self.mTargetName = target_name
        self.mType       = type
        self.mColour     = colour
        self.mLineStyle  = line_style
        self.mPriority   = priority
        self.mEdgeClass  = edge_class
                  
    def Write( self, outfile ):
        """write node to outputfile."""

        statement = ' sourcename: "%s" targetname: "%s" ' % (self.sourceName, self.mTargetName)
        if self.mColour: statement += ' color: %s' % self.mColour
        if self.mLineStyle: statement += ' linestyle: %s' % self.mLineStyle
        if self.mPriority: statement += ' priority: %i' % self.mPriority
        if self.mEdgeClass: statement += ' class: %i' % self.mEdgeClass                
        
        outfile.write( '\t%s: { ' % self.mType + statement + '}\n')
        
##-------------------------------------------------------------------------------
class Graph:

    COLOURS = ('black', 'blue', 'red', 'green', 'yellow', 'cyan', 'magenta',
               'darkblue','lightblue','aquamarine','darkred','lightred','khaki',
               'darkgreen','lightgreen','purple','darkyellow','lightyellow','yellowgreen',
               'darkmagenta','lightmagenta','pink','darkcyan','lightcyan','orange',
               'gold','lilac','orchid','darkgrey','lightgrey','turquoise','white')

    def __init__(self, format = None):
        self.mFormat = format
        self.mEdges = []
        self.mNodes = []

    def Write( self, outfile ):
        """write graph to outputfile."""
        outfile.write( 'graph: {\n')
        if self.mFormat:
            outfile.write( self.mFormat + "\n" )

        for node in self.mNodes:
            node.Write( outfile )
        for edge in self.mEdges:
            edge.Write( outfile )

        outfile.write( '}\n' )

    def AddNode ( self, node ):
        self.mNodes.append(node)

    def AddEdge(self,edge):
        self.mEdges.append(edge)
        
    def Read( self, infile ):
        """read graph from inputfile."""

        self.mNodes = []
        self.mEdges = []
        
        for line in infile.readlines():
            pass
        
    










################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
matrix2tree.py - build tree from a distance matrix
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python matrix2tree.py --help

Type::

   python matrix2tree.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import subprocess
import random

from types import *

import CGAT.Experiment as E
import CGAT.TreeTools as TreeTools
import CGAT.Tree as Tree
import CGAT.IOTools as IOTools
import CGAT.WrapperPhylip as WrapperPhylip

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: matrix2tree.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-i", "--invert-map", dest="invert_map", action="store_true",
                      help="""invert map.""")

    parser.add_option( "--input-format", dest="input_format", type="choice",
                       choices=("phylip", "full") ,
                       help="""input format.""")

    parser.add_option("-t", "--filename-tree", dest="filename_tree", type="string",
                      help="""filename with tree to fit.""")

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices=("nj", "kitsch", "fitch"),
                      help="""algorithm to run.""")

    parser.add_option("-e", "--replicates", dest="replicates", action="store_true",
                      help="replicates."  )

    parser.add_option("-r", "--root", dest="root", action="store_true",
                      help="midpoint root (if it is not rooted)."  )

    parser.add_option("-u", "--unroot", dest="unroot", action="store_true",
                      help="unroot tree (if it is rooted)."  )

    parser.add_option( "--skip-separators", dest="write_separators", action="store_false",
                      help="do not echo separators (starting with >)")
    
#    parser.add_option("-i", "--iterations", dest="iterations", type="int",
#                      help="number of iterations." )

    parser.add_option("-p", "--power", dest="power", type="float",
                      help="power." )

    parser.add_option( "--prune-tree", dest="prune_tree", action="store_true",
                       help="prune tree such to include only taxa which are part of the input matrix.")

    parser.add_option( "--add-random", dest="add_random", action="store_true",
                       help="add small random value to off-diagonal zero elements in matrix.")

    parser.add_option( "--pseudo-replicates", dest="pseudo_replicates", action="store_true",
                       help="add small random value to off-diagonal zero elements in matrix, even if they have no replicates.")

    parser.add_option( "--debug", dest="debug", action="store_true",
                       help="dump debug information." )


    parser.set_defaults(
        value = 0,
        method = "nj",
        input_format = "phylip",
        filename_tree = None,
        outgroup = None,
        replicates = False,
        root = False,
        unroot = False,
        power = 0,
        write_separators = True,
        prune_tree = False,
        add_random = False,
        debug = False,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    phylip = WrapperPhylip.Phylip()

    if options.debug:
        phylip.setLogLevel( options.loglevel )
    
    phylip.setPruneTree( options.prune_tree )

    lines = filter( lambda x: x[0] != "#", sys.stdin.readlines())

    chunks = filter( lambda x: lines[x][0] == ">", range(len(lines)) )

    if not chunks:
        options.write_separators = False
        chunks = [-1]
        
    chunks.append( len(lines) )

    for x in range(len(chunks) -1 ):
        
        matrix = lines[chunks[x]+1:chunks[x+1]]

        ## parse phylip matrix
        if options.add_random:
            mm = []
            ids = []
            for l in range(1, len(matrix)):
                values = re.split("\s+", matrix[l][:-1])
                ids.append( values[0] )
                mm.append( map( lambda x: x.strip(), values[1:] ) )
                
            d = len(mm)
            if options.replicates:
                for row in range(d-1):
                    for col in range(row + 1, d):
                        cc = col * 2
                        rr = row * 2
                        if mm[row][cc] == "0" and mm[row][cc+1] != "0":
                            mm[row][cc+1] = "1"
                            mm[col][rr+1] = "1"
                            v = str(random.random() / 10000.0 )
                            mm[row][cc] = v
                            mm[col][rr] = v

            else:
                for row in range(d-1):
                    for col in range(row + 1, d):
                        if mm[row][col] == "0":
                            v = str(random.random() / 10000.0 )
                            mm[row][col] = v
                            mm[col][row] = v
                    
            matrix = ["%i\n" % d]
            for row in range(d):
                matrix.append( ids[row] + "    " + "    ".join(mm[row]) + "\n" )

        ## parse phylip matrix
        if options.pseudo_replicates:
            mm = []
            ids = []
            for l in range(1, len(matrix)):
                values = re.split("\s+", matrix[l][:-1])
                ids.append( values[0] )
                mm.append( map( lambda x: x.strip(), values[1:] ) )
                
            d = len(mm)
            if options.replicates:
                for row in range(d-1):
                    for col in range(row + 1, d):
                        cc = col * 2
                        rr = row * 2
                        if mm[row][cc+1] == "0":
                            mm[row][cc+1] = "1"
                            mm[col][rr+1] = "1"
                            v = str(random.random() / 10000.0 )
                            mm[row][cc] = v
                            mm[col][rr] = v
                        else:
                            mm[row][cc+1] = "100"
                            mm[col][rr+1] = "100"
            else:
                for row in range(d-1):
                    for col in range(row + 1, d):
                        if mm[row][col] == "0":
                            v = str(random.random() / 10000.0 )
                            mm[row][col] = v
                            mm[col][row] = v
                    
            matrix = ["%i\n" % d]
            for row in range(d):
                matrix.append( ids[row] + "    " + "    ".join(mm[row]) + "\n" )

        phylip.setMatrix( matrix )
        
        phylip_options = []

        if options.filename_tree:
            nexus = TreeTools.Newick2Nexus( open(options.filename_tree, "r") )
            ref_tree = nexus.trees[0]
            phylip.setTree( ref_tree )
            phylip_options.append( "U" )
        else:
            ref_tree = None
            
        if options.method == "nj":
            phylip.setProgram( "neighbor")
            
        elif options.method == "fitch":
            phylip.setProgram( "fitch")
            
        elif options.method == "kitsch":        
            phylip.setProgram( "kitsch")

        if options.replicates:
            phylip_options.append("S")

        if options.power > 0:
            phylip_options.append("P")
            phylip_options.append("%f" % options.power)        

        phylip_options.append("Y")        

        phylip.setOptions( phylip_options )

        result = phylip.run()

        ## root with outgroup
        if options.root:        
            if options.outgroup:
                pass
            # midpoint root
            else:
                for tree in result.mNexus.trees:
                    tree.root_midpoint()
                    
        ## explicitely unroot
        elif options.unroot:
            phylip.setOptions( ("Y", "W", "U", "Q" ) )
            phylip.setProgram( "retree" )
            for x in range( len(result.mNexus.trees)):
                phylip.setTree( result.mNexus.trees[x])
                xresult = phylip.run()
                result.mNexus.trees[x] = xresult.mNexus.trees[0]

        if options.write_separators:
            options.stdout.write( lines[chunks[x]] )

        if result.mNexus:
            options.stdout.write( TreeTools.Nexus2Newick( result.mNexus ) + "\n" )

        if options.loglevel >= 1:
            if ref_tree:
                nref = len(ref_tree.get_terminals() )
            else:
                nref = 0
            for tree in result.mNexus.trees:
                options.stdlog.write("# ninput=%i, nreference=%i, noutput=%i\n" % ( len(matrix)-1, nref, len(tree.get_terminals()) ) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
set_diff.py - compare contents of several files
===============================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

pairwise comparison of multiple sets. 

The script outputs a tab-separated table with the following fields:

set1, set2: labels of sets that are compared
n1, n2: number of elements in the two sets
union: size of union of both sets
inter: size of the intersection of two sets
unique1: number of unique elements in set1
unique2: number of unique elements in set2

If the options --add-percent is given, the following columns are added:
pinter: intersection / union
punique1: unique1 / n1
punique1: unique2 / n2
pcov1: coverage of set1 by set2 = intersection / n1
pcov2: coverage of set2 by set1 intersection / n2
pcovmax: maximum coverage of either set = max(pcov1, pcov2)

Usage
-----

Example::

   python set_diff.py --help

Type::

   python set_diff.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

if __name__ == "__main__":
    
    parser = E.OptionParser( version = "%prog version: $Id: set_diff.py 2782 2009-09-10 11:40:29Z andreas $" )

    parser.add_option("-p", "--add-percent", dest="add_percent", action="store_true",
                      help="add percentage information to each line." )

    parser.add_option("-t", "--headers", dest="headers", type="string",
                      help="comma separated list of headers. If empty or set to '-', filenames are used." )

    parser.add_option("--skip-header", dest="add_header", action="store_false",
                      help="do not add header to flat format."  )

    parser.add_option("--write-header", dest="write_header", action="store_true",
                      help="write header and exit."  )

    parser.add_option("--with-title", dest="with_title", action="store_true",
                      help="use column titles in input data [%default]."  )

    parser.add_option("--no-title", dest="with_title", action="store_false",
                      help="there are no titles in input data [%default]."  )

    parser.set_defaults(
        add_percent = False,
        percent_format = "%5.2f",
        headers = None,
        add_header = True,
        write_header = False,
        with_title = True,
        )

    (options, args) = E.Start( parser )

    if options.add_header:
        options.stdout.write("set1\tset2\tn1\tn2\tunion\tinter\tunique1\tunique2" )
        if options.add_percent:
            options.stdout.write( "\tpinter\tpunique1\tpunique2\tpcov1\tpcov2\tpcovmax" )
        options.stdout.write("\n")

        if options.write_header:
            sys.exit(0)
            
    if len(args) < 2:
        raise ValueError( "please supply at least two filenames.")

    headers, titles, sets = [], [], []

    if options.headers:
        if options.headers == "-":
            headers=args
        else:
            headers=options.headers.split(",")
            if len(headers) != len(args):
                raise ValueError ("please supply the same number of headers as there are filenames." )

    for f in args:
        if options.with_title:
            title, data = IOTools.readList( open(f,"r"), with_title = options.with_title )
            titles.append( title )
        else:
            data = IOTools.readList( open(f,"r") )
        sets.append( set( data ))
        
    if not headers and titles:
        headers = titles
    else:
        headers = args

    for x in range(len(sets)-1):
        set1=sets[x]

        for y in range(x+1, len(sets)):
            set2=sets[y]
            l1,l2 = len(set1), len(set2)
            options.stdout.write("%s\t%s\t%i\t%i\t%i\t%i\t%i\t%i" % (headers[x],headers[y],
                                                                     l1,l2,
                                                                     len(set1.union(set2)),                                                                   
                                                                     len(set1.intersection(set2)),
                                                                     len(set1.difference(set2)),
                                                                     len(set2.difference(set1))))



            
            if options.add_percent:
                if len(set1) == 0:
                    ri, r1, r2 = 0, 1, 0
                    c1, c2, cm = 1, 0, 0
                elif len(set2) == 0:
                    ri, r1, r2 = 0, 0, 1
                    c1, c2, cm = 0, 1, 0
                else:
                    i = len(set1.intersection(set2))
                    ri, r1, r2 = (
                        i / float(len(set1.union(set2))),
                        len(set1.difference(set2)) / float(l1),
                        len(set2.difference(set1)) / float(l2))
                    c1, c2 = (i / float(l1), i / float(l2))
                    cm = max( c1, c2)

                options.stdout.write( "\t" + ("\t".join([options.percent_format for z in range(6)] )) % (ri, r1, r2, c1, c2, cm))
            
            options.stdout.write("\n")
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
graph_filter_links_redundant.py -
=============================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

remove from a sorted list of links those which are redundant.

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

Code
----

'''

import os
import sys
import re
import optparse

import CGAT.Experiment as E

class Map:

    def __init__(self ):
        (self.mQueryToken, self.mSbjctToken, self.score,
         self.mQueryFrom, self.mQueryTo, self.mQueryAli,
         self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli ) =\
         ("", "", 0, 0, 0, "", 0, 0, "" )
        self.mIsExpanded = False
        self.mMapQuery2Sbjct = None

    def Read( self, line ):
        
        (self.mQueryToken, self.mSbjctToken, self.score,
         self.mQueryFrom, self.mQueryTo, self.mQueryAli,
         self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli) = line[:-1].split("\t")[:9]

        (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo) = map(
            int, (self.mQueryFrom, self.mQueryTo, self.mSbjctFrom, self.mSbjctTo))

        self.score = float(self.score)

    def __str__( self ):

        return string.join( map(str, (
            self.mQueryToken, self.mSbjctToken, self.score,
            self.mQueryFrom, self.mQueryTo, self.mQueryAli,
            self.mSbjctFrom, self.mSbjctTo, self.mSbjctAli)), "\t")

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-t", "--test", dest="test", type="string",
                      help="supply help"  )


    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    last_m = None

    for line in options.stdin:
        
        if line[0] == "#": continue

        m = Map()
        m.Read( line )

        if not last_m:
            last_m = m
            continue
        
        if m.mQueryToken != last_m.mQueryToken or \
           m.mSbjctToken != last_m.mSbjctToken or \
           m.mQueryFrom > last_m.mQueryTo:
            print str(last_m)
            last_m = m
        else:
            if m.mQueryTo - m.mQueryFrom > last_m.mQueryTo - last_m.mQueryFrom:
                last_m = m

    print str(last_m)
        
    ## write footer and output benchmark information.
    E.Stop()


if __name__ == "__main__":
    sys.exit( main( sys.argv) )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
runExpression.py - wrap various differential expression tools
=============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script provides a convenience wrapper for differential expression analysis 
for a variety of methods.

Methods implemented are:

   DESeq
   EdgeR

The aim of this script is to provide a common tabular output format that is consistent
between the different methods. The columns in the output are:

+--------------+------------------------------------------------------+
|*Column name* |*Content*                                             |
+--------------+------------------------------------------------------+
|test_id       |Name of the test (gene name, ...                      |
+--------------+------------------------------------------------------+
|treatment_name|Name of the treatment condition                       |
+--------------+------------------------------------------------------+
|treatment_mean|Estimated expression value for treatment              |
+--------------+------------------------------------------------------+
|treatment_std |Standard deviation                                    |
+--------------+------------------------------------------------------+
|control_name  |Name of the control condition                         |
+--------------+------------------------------------------------------+
|control_mean  |Estimated expression value for control                |
+--------------+------------------------------------------------------+
|control_std   |Standard deviation                                    |
+--------------+------------------------------------------------------+
|pvalue        |The p value for rejecting the null hypothesis         |
+--------------+------------------------------------------------------+
|qvalue        |Multiple testing correction                           |
+--------------+------------------------------------------------------+
|l2fold        |log2 foldchange of treatment/control                  |
+--------------+------------------------------------------------------+
|fold          |foldchange of treatment/control                       |
+--------------+------------------------------------------------------+
|significant   |Flag, 1 if test called significant according to FDR   |
+--------------+------------------------------------------------------+
|status        |test status (OK|FAIL)                                 |
+--------------+------------------------------------------------------+

The script will call each of the method and output a variety of diagnostic plots.

Usage
-----

Documentation
-------------

Command line options
--------------------

'''

import math
import numpy
import sys, os, optparse
import collections
import itertools

from rpy2.robjects import r as R
import rpy2.robjects as ro
import rpy2.robjects.numpy2ri

try:
    import CGAT.Experiment as E
    import CGAT.Pipeline as P
    import CGAT.Database as Database
    import CGAT.IOTools as IOTools
    import CGAT.Stats as Stats
    import CGAT.Expression as Expression
except ImportError:
    import Experiment as E
    import Pipeline as P
    import Database,IOTools,Stats
    import Expression

import sqlite3

try:
    PARAMS = P.getParameters()
except IOError:
    pass


def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-t", "--filename-tags", dest="input_filename_tags", type="string",
                      help="input file with tag counts [default=%default]."  )

    parser.add_option("-d", "--filename-design", dest="input_filename_design", type="string",
                      help="input file with experimental design [default=%default]."  )

    parser.add_option("-o", "--outfile", dest="output_filename", type="string",
                      help="output filename [default=%default]."  )

    parser.add_option("-m", "--method", dest="method", type="choice",
                      choices = ("deseq", "edger", "cuffdiff", "summary" ),
                      help="differential expression method to apply [default=%default]."  )

    parser.add_option( "--deseq-dispersion-method", dest="deseq_dispersion_method", type="choice",
                      choices = ("pooled", "per-condition", "blind"),
                      help="dispersion method for deseq [default=%default]."  )

    parser.add_option( "--deseq-fit-type", dest="deseq_fit_type", type="choice",
                      choices = ("parametric", "local"),
                      help="fit type for deseq [default=%default]."  )

    parser.add_option("-f", "--fdr", dest="fdr", type="float",
                      help="fdr to apply [default=%default]."  )

    parser.add_option("-r","--reference-group", dest="ref_group", type="string",
                      help="Group to use as reference to compute fold changes against [default=$default]")


    parser.set_defaults(
        input_filename_tags = "-",
        input_filename_design = None,
        output_filename = sys.stdout,
        method = "deseq",
        fdr = 0.1,
        deseq_dispersion_method = "pooled",
        deseq_fit_type = "local",
        ref_group = None
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv, add_output_options = True )

    if options.input_filename_tags == "-":
        fh = P.getTempFile()
        fh.write( "".join( [ x for x in options.stdin ] ) )
        fh.close()
        options.input_filename_tags = fh.name
    else:
        fh = None

    if options.method == "deseq":
        assert options.input_filename_tags and os.path.exists(options.input_filename_tags)
        assert options.input_filename_design and os.path.exists(options.input_filename_design)
        Expression.runDESeq( options.input_filename_tags,
                             options.input_filename_design,
                             options.output_filename,
                             options.output_filename_pattern,
                             fdr = options.fdr,
                             dispersion_method = options.deseq_dispersion_method,
                             fit_type = options.deseq_fit_type,
                             ref_group = options.ref_group)

    elif options.method == "edger":
        assert options.input_filename_tags and os.path.exists(options.input_filename_tags)
        assert options.input_filename_design and os.path.exists(options.input_filename_design)
        Expression.runEdgeR( options.input_filename_tags,
                             options.input_filename_design,
                             options.output_filename,
                             options.output_filename_pattern,
                             fdr = options.fdr,
                             ref_group = options.ref_group)

    elif options.method == "summary":
        assert options.input_filename_tags and os.path.exists(options.input_filename_tags)
        Expression.outputTagSummary( options.input_filename_tags,
                                     options.stdout,
                                     options.output_filename_pattern )

    if fh and os.path.exists( fh.name): os.unlink( fh.name )

    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
align_all_vs_all.py - all-vs-all pairwise alignment
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script computes all-vs-all alignments between
sequences in a :term:`fasta` formatted file.

Currently only Smith-Waterman protein alignment is
implemented.

Usage
-----

Example::

   python align_all_vs_all.py --help

Type::

   python align_all_vs_all.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

import CGAT.Experiment as E

import alignlib
import CGAT.FastaIterator as FastaIterator

""" program $Id: align_all_vs_all.py 2782 2009-09-10 11:40:29Z andreas $
"""
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: align_all_vs_all.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-s", "--sequences", dest="filename_sequences", type="string",
                      help="input file with sequences"  )

    parser.set_defaults(
        filename_sequences = None,
        gop = -10.0,
        gep = -1.0,
        )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if options.filename_sequences:
        infile = open(options.filename_sequences, "r")
    else:
        infile = sys.stdin

    parser = FastaIterator.FastaIterator( infile )

    sequences = []
    while 1:
        cur_record = iterator.next()
        
        if cur_record is None: break
        sequences.append( (cur_record.title, alignlib.makeSequence(re.sub( " ", "", cur_record.sequence)) ) )
    
    if options.filename_sequences:
        infile.close()

    alignator = alignlib.makeAlignatorFullDP( options.gop, options.gep )
    map_a2b = alignlib.makeAlignataVector()
    nsequences = len(sequences)
    
    for x in range(0,nsequences-1):
        for y in range(x+1, nsequences):
            alignator.Align( sequences[x][1], sequences[y][1], map_a2b)

            row_ali, col_ali = alignlib.writeAlignataCompressed( map_a2b )
            
            options.stdout.write( "%s\t%s\t%i\t%i\t%i\t%s\t%i\t%i\t%s\t%i\t%i\t%i\t%i\n" % (\
                sequences[x][0], sequences[y][0],
                map_a2b.getScore(),
                map_a2b.getRowFrom(),
                map_a2b.getRowTo(),
                row_ali,
                map_a2b.getColFrom(),
                map_a2b.getColTo(),
                col_ali,
                map_a2b.getScore(),
                100 * alignlib.calculatePercentIdentity( map_a2b, sequences[x][1], sequences[y][1]),
                sequences[x][1].getLength(),
                sequences[y][1].getLength() ))
            

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
blast2fasta.py - 
======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

.. todo::
   
   describe purpose of the script.

Usage
-----

Example::

   python blast2fasta.py --help

Type::

   python blast2fasta.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import tempfile
import time
import optparse
import math

import CGAT.Experiment as E
import CGAT.Genomics as Genomics
import CGAT.BlastAlignments as BlastAlignments
import alignlib

USAGE="""python %s [OPTIONS] < graph.in > graph.out

Version: $Id: blast2fasta.py 2782 2009-09-10 11:40:29Z andreas $

Convert a blast graph into a pairwise alignment graph

Options:
-h, --help                      print this message.
-v, --verbose=                  loglevel.
-p, --peptides=                 filename with peptide sequences
""" % sys.argv[0]

if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: blast2fasta.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option("-s", "--sequences", dest="filename_sequences", type="string",
                      help="filename with sequences."  )
    parser.add_option("-f", "--format", dest="format", type="string",
                      help="output format."  )
    
    parser.set_defaults(
        filename_sequences = None,
        format = "fasta",
        )

    (options, args) = E.Start( parser )

    if not options.filename_sequences:
        raise "please supply filename with sequences."

    sequences = Genomics.ReadPeptideSequences( open(options.filename_sequences, "r") )

    if options.loglevel >= 1:
        print "# read %i sequences" % len(sequences)
        
    for k in sequences.keys():
        sequences[k] = alignlib.makeSequence( sequences[k] )

    if options.loglevel >= 2:
        print "# converted %i sequences" % len(sequences)
    
    ninput, noutput, nskipped, nfailed = 0, 0, 0, 0
    link = BlastAlignments.Link()

    ali = alignlib.makeAlignataVector()
    
    for line in sys.stdin:
        
        if line[0] == "#": continue

        link.Read( line )
        ninput += 1

        if link.mQueryToken not in sequences or link.mSbjctToken not in sequences:
            nskipped += 1
            continue
        
        ali.Clear()
        alignlib.fillAlignataCompressed( ali, link.mQueryFrom, link.mQueryAli, link.mSbjctFrom, link.mSbjctAli )


        result = alignlib.writePairAlignment( sequences[link.mQueryToken], sequences[link.mSbjctToken], ali ).split("\n")

        if len(result) != 3:
            nfailed += 1

        if options.format == "fasta":
            print ">%s %i-%i\n%s\n>%s %i-%i\n%s\n" %\
                  (link.mQueryToken, link.mQueryFrom, link.mQueryTo, result[0].split("\t")[1],
                   link.mSbjctToken, link.mSbjctFrom, link.mSbjctTo, result[1].split("\t")[1] )
            
        noutput += 1
        
    print "# ninput=%i, noutput=%i, nskipped=%i, nfailed=%i" % (ninput, noutput, nskipped, nfailed)
    E.Stop()

            
    



"""
Iterators.py - general purpose iterators.
==========================================

:Author: Unknown
:Release: $Id$
:Date: |today|
:Tags: Python

A collection of useful, general purpose iterators.

This code was downloaded from an unknown source.
"""

import random

##------------------------------------------------------------
def sample(iterable, sample_size = None):
    """sample # copies from iterator without replacement.

    Stores a temporay copy of the items in iterable. The function
    has thus a possibly high memory footprint and long pre-processing
    time to yield the first element.

    If sample_size is not given, the iterator returns elements in
    random order ( see random.shuffle() )
    """

    saved = []
    for element in iterable: saved.append(element)
        
    random.shuffle(saved)
    if not sample_size: sample_size=len(saved)
    for x in xrange( sample_size ):
        yield saved[x]


def group_by_distance( iterable, distance = 1 ):
    """group integers into non-overlapping intervals that
    are at most *distance* apart.

    >>> list( group_by_distance( (1,1,2,4,5,7) ) )
    [(1, 3), (4, 6), (7, 8)]

    >>> list( group_by_distance( [] ) )
    []

    >>> list( group_by_distance( [3] ) )
    [(3, 4)]

    >>> list( group_by_distance( [3,2] ) )
    Traceback (most recent call last):
    ...
    ValueError: iterable is not sorted: 2 < 3
    """
    i = iter(iterable)
    end = None
    start = end = cur = i.next()

    for cur in i:
        if cur < end: raise ValueError( "iterable is not sorted: %i < %i" % (cur,end) )
        if cur - end > distance:
            yield (start,end+1)
            start = cur
        end = cur
    yield (start,end+1)

if __name__ == "__main__":
    import doctest
    doctest.testmod()
 
